/**
 *
 * Copyright 2005-2006 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.logicblaze.soa;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;

import javax.sql.DataSource;

import org.apache.ddlutils.Platform;
import org.apache.ddlutils.PlatformFactory;
import org.apache.ddlutils.io.DatabaseIO;
import org.apache.ddlutils.model.Database;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.core.io.Resource;

public class SchemaCreator implements InitializingBean {

	private Platform platform;
	private Database database;
    private Resource databaseResource;
    private String sqlScript;
    private Resource sqlScriptResource;
	private DataSource dataSource;
	private boolean updateSchema = true;
	private boolean doDrops = true;
	private boolean modifyColums = true;
	private boolean continueOnError = false;
	
	public void afterPropertiesSet() throws Exception {
		if (dataSource == null) {
			throw new NullPointerException("dataSource is null");
		}
        platform = PlatformFactory.createNewPlatformInstance(dataSource);
		// Create database
		if (database == null && databaseResource != null) {
			InputStream is = databaseResource.getInputStream();
			try {
				DatabaseIO io = new DatabaseIO();
				io.setValidateXml(false);
				database = io.read(new InputStreamReader(is));
			} finally {
				is.close();
			}
		}
		if (database != null) {
	        if (isUpdateSchema()) {
	        	platform.alterTables(database, doDrops, modifyColums, continueOnError);
	        } else {
	        	platform.createTables(database, false, continueOnError);
	        }
		}
		// Run sql
		if (sqlScript == null && sqlScriptResource != null) {
			InputStream is = sqlScriptResource.getInputStream();
			try {
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				byte[] buffer = new byte[1024];
				int length;
				while ((length = is.read(buffer)) > 0) {
					baos.write(buffer, 0, length);
				}
				sqlScript = baos.toString();
			} finally {
				is.close();
			}
		}
		if (sqlScript != null) {
			platform.evaluateBatch(sqlScript, continueOnError);
		}
	}

	public Database getDatabase() {
		return database;
	}

	public void setDatabase(Database database) {
		this.database = database;
	}

	public DataSource getDataSource() {
		return dataSource;
	}

	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	public boolean isUpdateSchema() {
		return updateSchema;
	}

	public void setUpdateSchema(boolean updateSchema) {
		this.updateSchema = updateSchema;
	}

	public Resource getDatabaseResource() {
		return databaseResource;
	}

	public void setDatabaseResource(Resource databaseUrl) {
		this.databaseResource = databaseUrl;
	}

	public Resource getSqlScriptResource() {
		return sqlScriptResource;
	}

	public void setSqlScriptResource(Resource sqlResource) {
		this.sqlScriptResource = sqlResource;
	}

	public String getSqlScript() {
		return sqlScript;
	}

	public void setSqlScript(String sqlScript) {
		this.sqlScript = sqlScript;
	}

	public boolean isContinueOnError() {
		return continueOnError;
	}

	public void setContinueOnError(boolean continueOnError) {
		this.continueOnError = continueOnError;
	}

}
