/**
 *
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 **/

package org.logicblaze.ldap;

import com.liferay.portal.NoSuchUserException;
import com.liferay.portal.model.User;
import com.liferay.portal.security.auth.AuthException;
import com.liferay.portal.security.auth.Authenticator;
import com.liferay.portal.security.pwd.PwdEncryptor;
import com.liferay.portal.service.spring.UserLocalServiceUtil;
import com.liferay.portal.util.PrefsPropsUtil;
import com.liferay.portal.util.PropsUtil;

import org.logicblaze.ldap.bean.LDAPServerBean;
import org.logicblaze.ldap.bean.UserBean;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.naming.*;
import javax.naming.directory.*;
import java.util.Date;
import java.util.Locale;

public class LDAPAuth implements Authenticator {

    private static final Log log = LogFactory.getLog(LDAPAuth.class);

    public int authenticateByEmailAddress(String companyId, String emailAddress, String password)
            throws AuthException {

        int isSuccess = SUCCESS;

        DirContext context = null;

        try {

            //LDAPServerBean serverInfo = (LDAPServerBean) new InitialContext().lookup("java:comp/env/ldap/apacheDS");
           LDAPServerBean serverInfo =  UserServiceUtil.getLDAPServerBean(); 


            context = UserServiceUtil.getDirContext(serverInfo);

            UserBean userInfo = this.getAuthenticatedUserBean(context, serverInfo.getUserBase(),
                                                    emailAddress, PwdEncryptor.encrypt(password));

            if (userInfo != null) {

                User user = null;

                try {

                    user = UserLocalServiceUtil.getUserByEmailAddress(companyId, emailAddress);
                } catch (NoSuchUserException nsue) {

                    // Grab attributes from LDAP
                    //Note: userInfo was populated during authentication.

                    boolean autoUserId = false;
                    String userId = userInfo.getUserID();
                    boolean autoPassword = false;
                    String password1 = password;
                    String password2 = password;
                    boolean passwordReset = false;
                    String firstName = (String) userInfo.getGivenname();
                    String middleName = "";
                    String lastName = (String) userInfo.getSurname();
                    String nickName = "";
                    boolean male = true;
                    Date birthday = new Date();
                    Locale locale = Locale.US;
                    String prefixId = "";
                    String suffixId = "";
                    int birthdayMonth = 0;
                    int birthdayDay = 0;
                    int birthdayYear = 0;
                    String jobTitle = "";
                    String organizationalId = "";
                    String locationId = "";

                    user = UserLocalServiceUtil.addUser(userId,companyId, autoUserId, userId, autoPassword,
                            password1, password2, passwordReset, emailAddress,
                            locale, firstName, middleName, lastName, nickName,
                            prefixId, suffixId, male, birthdayMonth, birthdayDay,
                            birthdayYear, jobTitle, organizationalId, locationId);
                   

                }
            } else {
                isSuccess = FAILURE;

            }
            return isSuccess;

        } catch (Exception e) {
            throw new AuthException(e);

        } finally {
            UserServiceUtil.closeDirContext(context);

        }
    }

    public int authenticateByUserId(String companyId, String userId, String password) throws AuthException {
        // Empty implementation;
        return SUCCESS;
    }

    protected UserBean getAuthenticatedUserBean(DirContext context, String userBase, String email, String password)
            throws NamingException {

        String searchContext = userBase;

        Attribute objclass = new BasicAttribute("objectClass");
        Attribute mail = new BasicAttribute("mail");

        //adding attribute values
        objclass.add("person");
        mail.add(email);

        //Instantiate Attributes object and put search attributes in it.
        Attributes attrs = new BasicAttributes();
        attrs.put(objclass);
        attrs.put(mail);

        // ----------------------------------------
        // Executing search
        //------------------------------------------
        NamingEnumeration results = context.search(searchContext, attrs);

        if (results == null || !results.hasMore()) {
            return null;
        }

        SearchResult result = (SearchResult) results.next();

        if (results.hasMore()) {
            //ignore for now
        }

        NameParser parser = context.getNameParser("");
        Name contextName = parser.parse(context.getNameInNamespace());
        Name baseName = parser.parse(userBase);
        Name entryName = parser.parse(result.getName());
        Name name = contextName.addAll(baseName);
        name = name.addAll(entryName);
        String dn = name.toString();

        if (bindUser(context, dn, password)) {
            return populateUserBean(result);

        }
        return null;
    }

    protected String getAuthenticatedUserId(DirContext context, String userBase, String email, String password)
            throws NamingException {

        String searchContext = userBase;

        Attribute objclass = new BasicAttribute("objectClass");
        Attribute mail = new BasicAttribute("mail");

        //adding attribute values
        objclass.add("person");
        mail.add(email);

        //Instantiate Attributes object and put search attributes in it.
        Attributes attrs = new BasicAttributes();
        attrs.put(objclass);
        attrs.put(mail);

        // ----------------------------------------
        // Executing search
        //------------------------------------------

        NamingEnumeration results = context.search(searchContext, attrs);

        if (results == null || !results.hasMore()) {
            return null;
        }

        SearchResult result = (SearchResult) results.next();

        if (results.hasMore()) {
            //ignore for now
        }

        NameParser parser = context.getNameParser("");
        Name contextName = parser.parse(context.getNameInNamespace());
        Name baseName = parser.parse(userBase);
        Name entryName = parser.parse(result.getName());
        Name name = contextName.addAll(baseName);
        name = name.addAll(entryName);
        String dn = name.toString();

        if (bindUser(context, dn, password)) {
            return result.getName();
        }
        return null;
    }

    protected UserBean populateUserBean (SearchResult results) throws NamingException {

        UserBean userBean = new UserBean();
        Attributes resultsAttr = results.getAttributes();

        String value;
        if (resultsAttr != null) {
            // sn attribute
            Attribute attr = resultsAttr.get("sn");
            //System.out.println(attr.getID() + ":" + (attr.getAll()).next());
            //map.put(attr.getID(), (attr.getAll()).next());
            value = (String) (Object) attr.getAll().nextElement();
            userBean.setSurname(value);

            // givenname attribute
            attr = resultsAttr.get("givenname");
            //System.out.println(attr.getID() + ":" + (attr.getAll()).next());
            //map.put(attr.getID(), (attr.getAll()).next());
            value = (String) (Object) attr.getAll().nextElement();
            userBean.setGivenname(value);

            // mail
            attr = resultsAttr.get("mail");
            //System.out.println(attr.getID() + ":" + (attr.getAll()).next());
            //map.put(attr.getID(), (attr.getAll()).next());
            value = (String) (Object) attr.getAll().nextElement();
            userBean.setEmialAddress(value);

            // uid
            attr = resultsAttr.get("uid");
            //System.out.println(attr.getID() + ":" + (attr.getAll()).next());
            //map.put(attr.getID(), (attr.getAll()).next());
            value = (String) (Object) attr.getAll().nextElement();
            userBean.setUserID(value);
        }

        return userBean;
    }

    protected boolean bindUser(DirContext context, String dn, String password) throws NamingException {

        boolean isValid = false;

        context.addToEnvironment(Context.SECURITY_PRINCIPAL, dn);
        context.addToEnvironment(Context.SECURITY_CREDENTIALS, password);

        try {
            context.getAttributes("", null);
            isValid = true;

        } catch (AuthenticationException e) {
            isValid = false;
            log.info("Authentication failed: " + dn);
        }

        return isValid;
    }
}