/**
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.logicblaze.soa;

import java.io.InputStream;
import java.io.PrintStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xbean.server.main.KernelMain;

/**
 * The main entry point into the server called from within a valid classloader which has xbean and spring on it
 * 
 * @version $Revision: 66 $
 */
public class GeronimoBootstrapMain {

    private static final Log log = LogFactory.getLog(GeronimoBootstrapMain.class);
    
    GeronimoBootstrap bootstrap = new GeronimoBootstrap();
    boolean started=false;
    private KernelMain main;
    private Thread kernelThread;

    public GeronimoBootstrapMain() {
        bootstrap.setConfigurationFile("fuse-bootstrap.xml");
    }
    
    public static void main(String[] args) {
        GeronimoBootstrapMain main = new GeronimoBootstrapMain();
        main.main(args, System.in, System.out, null);
    }
    
    synchronized public void main(String[] args, InputStream in, PrintStream out, String baseDirectory) {
        if( args.length > 0  && "stop".equals(args[0]) ) {
            stop();
        } else {
            start(args, baseDirectory);
        }
    }

    /**
     * 
     */
    private void stop() {
        log.info("Stopping FUSE kernel: "+this);
        if( started && main!=null ) {
            main.destroy();
            try {
                log.info("Waiting for kernel to stop.");
                kernelThread.join();
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
            log.info("Kernel stopped.");
            bootstrap.getApplicationContext().close();
            log.info("Context closed.");
            started = false;
        }
    }

    /**
     * @param args
     * @param baseDirectory
     */
    private void start(String[] args, String baseDirectory) {
        if( started ) {
            throw new IllegalStateException("Already started");
        }
        
        log.info("Starting Fuse: "+this);
        bootstrap.setServerBaseDirectory(baseDirectory);
        bootstrap.initialize(args);
        main = (KernelMain) bootstrap.getMain();
        
        
        // Now run it async so that we can return to the caller.
        kernelThread = new Thread("Fuse Kernel") {
            public void run() {
                bootstrap.boot();
                log.info("Fuse Kernel Stopped.");
            };
        };
        kernelThread.start();
        
        started = true;
    }
        
}
