CREATE DATABASE IF NOT EXISTS inventoryDB;

USE inventoryDB;

CREATE TABLE IF NOT EXISTS transactions (
  id         INT UNSIGNED         NOT NULL  AUTO_INCREMENT,
  tx_date    TIMESTAMP            NOT NULL  DEFAULT CURRENT_TIMESTAMP,
  tx_type    ENUM('buy', 'sell')  NOT NULL,
  item_name  VARCHAR(100)         NOT NULL,
  user_name  VARCHAR(100)         NULL      DEFAULT 'unknown',
  quantity   INT                  NOT NULL  DEFAULT 0,
  PRIMARY KEY  (id)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE IF NOT EXISTS items (
  id           INT UNSIGNED NOT NULL  AUTO_INCREMENT,
  item_name    VARCHAR(100) NOT NULL,
  supplied_by  VARCHAR(100) NOT NULL  DEFAULT 'unknown',
  curr_stock   INT          NOT NULL  DEFAULT 0,
  item_price   FLOAT(7,2)   NOT NULL  DEFAULT 0.0,
  last_update  TIMESTAMP    NOT NULL  DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  PRIMARY KEY (id),
  UNIQUE (item_name)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

INSERT IGNORE INTO items (item_name,supplied_by,curr_stock,item_price) VALUES ("SOAP POWDER",   "Good Life Supplier",  2100, 2.50);
INSERT IGNORE INTO items (item_name,supplied_by,curr_stock,item_price) VALUES ("DOG FOOD",      "Cool Stuff Supplier", 1400, 17.80);
INSERT IGNORE INTO items (item_name,supplied_by,curr_stock,item_price) VALUES ("KITCHEN TOWEL", "Cool Stuff Supplier", 1600, 8.00);
INSERT IGNORE INTO items (item_name,supplied_by,curr_stock,item_price) VALUES ("LOTSA BREAD",   "Best Goods Supplier", 1950, 7.99);
INSERT IGNORE INTO items (item_name,supplied_by,curr_stock,item_price) VALUES ("FISHY FINGERS", "Good Life Supplier",  2700, 11.01);
INSERT IGNORE INTO items (item_name,supplied_by,curr_stock,item_price) VALUES ("BAKED BEANS",   "Sweet Food Supplier", 3300, 10.99);
INSERT IGNORE INTO items (item_name,supplied_by,curr_stock,item_price) VALUES ("WOA MILK",      "Good Life Supplier",  1000, 5.75);

CREATE TRIGGER trans_trg AFTER INSERT ON transactions
FOR EACH ROW  
    UPDATE IGNORE items SET
        curr_stock = IF(NEW.tx_type="buy", curr_stock - NEW.quantity, curr_stock + NEW.quantity)
    WHERE
        NEW.item_name=item_name;
