/**
 * Copyright (c) 2000-2006 Liferay, LLC. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.admin.action;

import com.liferay.portal.security.auth.PrincipalException;
import com.liferay.portal.servlet.PortalSessionContext;
import com.liferay.portal.struts.PortletAction;
import com.liferay.portlet.admin.util.OmniadminUtil;
import com.liferay.util.ParamUtil;
import com.liferay.util.servlet.SessionErrors;
import com.liferay.util.servlet.SessionMessages;

import java.security.Principal;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletConfig;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * <a href="EditSessionAction.java.html"><b><i>View Source</i></b></a>
 *
 * @author  Brian Wing Shun Chan
 *
 */
public class EditSessionAction extends PortletAction {

	public void processAction(
			ActionMapping mapping, ActionForm form, PortletConfig config,
			ActionRequest req, ActionResponse res)
		throws Exception {

		// Workaround fix to get userId because req.getRemoteUser()
		// return the toString() of object Principal.
		String remoteUser = null;

		Principal principal = req.getUserPrincipal();
		if(principal != null){
		  remoteUser = principal.getName();
	    }

		if (!OmniadminUtil.isOmniadmin(remoteUser)) {
			SessionErrors.add(req, PrincipalException.class.getName());

			setForward(req, "portlet.admin.error");

			return;
		}

		invalidateSession(req, res);
	}

	public ActionForward render(
			ActionMapping mapping, ActionForm form, PortletConfig config,
			RenderRequest req, RenderResponse res)
		throws Exception {

		return mapping.findForward(
			getForward(req, "portlet.admin.edit_session"));
	}

	protected void invalidateSession(ActionRequest req, ActionResponse res)
		throws Exception {

		String userTrackerId = ParamUtil.getString(req, "userTrackerId");

		HttpSession userSession = PortalSessionContext.get(userTrackerId);

		if (userSession != null) {
			try {
				if (!req.getPortletSession().getId().equals(userTrackerId)) {
					userSession.invalidate();
				}
			}
			catch (Exception e) {
				_log.error(e);
			}
		}

		// Session messages

		SessionMessages.add(req, "request_processed");

		// Send redirect

		res.sendRedirect(ParamUtil.getString(req, "redirect"));
	}

	private static Log _log = LogFactory.getLog(EditSessionAction.class);

}